package
{
	import flash.display.Graphics;
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.events.*;
	import flash.media.Microphone;
	import flash.system.Security;
	import flash.text.TextField;
	import flash.utils.ByteArray;
	import flash.utils.Endian;
	
	import ru.inspirit.analysis.*;
		
	public class Main extends Sprite
	{
		private var
			btnToggle:TextField,
			fft:FFT,
			fftAnalyzer:FFTSpectrumAnalyzer,
			isCapturing:Boolean = false,
			mic:Microphone,
			micData:ByteArray,
			PLOT_HEIGHT:uint,
			PLOT_WIDTH:uint,
			txtStatus:TextField;
		
		public function Main()
		{
			mic = Microphone.getMicrophone();
			Security.showSettings("2");
			if (mic) {
				PLOT_HEIGHT = stage.stageHeight;
				PLOT_WIDTH = stage.stageWidth;
				btnToggle = new TextField();
				btnToggle.text = "Toggle Microphone";
				btnToggle.selectable = false;
				btnToggle.addEventListener(MouseEvent.CLICK, toggleCapture);
				btnToggle.width = PLOT_WIDTH;
				txtStatus = new TextField();
				txtStatus.y = 20;
				txtStatus.width = PLOT_WIDTH;
				this.addChild(btnToggle);
				this.addChild(txtStatus);
				
				mic.rate = 44;
				mic.setSilenceLevel(10);
				fft = new FFT();
				fft.init(2048, 2);
				fftAnalyzer = new FFTSpectrumAnalyzer(fft, 44100);
				fftAnalyzer.initLogarithmicAverages(22, 2);
			}
		}
		
		private function onMicActivity(e:ActivityEvent):void
		{
			txtStatus.text = e.activating ? 'Activated' : 'Deactivated';
			if (!e.activating)
			{
				var g:Graphics = this.graphics;
				g.clear();
			}
		}
		
		private function onSampleData(e:SampleDataEvent):void
		{
			micData = new ByteArray();
			micData.endian = Endian.LITTLE_ENDIAN;
			
			while (e.data.bytesAvailable) {
				micData.writeFloat(e.data.readFloat());
			}
			micData.position = 0;
			fft.setStereoRAWDataByteArray(micData);
			fft.forwardFFT();
			
			var spectrumData:ByteArray = fftAnalyzer.analyzeSpectrum();
			renderSpectrum(spectrumData);
		}
		
		private function startCapture():void
		{
			txtStatus.text = 'Start capture';
			mic.addEventListener(ActivityEvent.ACTIVITY, onMicActivity);
			mic.addEventListener(SampleDataEvent.SAMPLE_DATA, onSampleData);
		}
		private function stopCapture():void
		{
			txtStatus.text = 'Stop capture';
			mic.removeEventListener(ActivityEvent.ACTIVITY, onMicActivity);
			mic.removeEventListener(SampleDataEvent.SAMPLE_DATA, onSampleData);
		}
		private function toggleCapture(e:MouseEvent):void
		{
			if (!mic) {
				txtStatus.text = 'No microphone detected';
				return;
			}
			
			if (isCapturing) {
				stopCapture();
			} else {
				startCapture();
			}
			
			isCapturing = !isCapturing;
		}
		
		private function renderSpectrum(bytes:ByteArray):void
		{
			var CHANNEL_LENGTH:uint = bytes.length / 2 >> 2;
			var BAND_WIDTH:uint = PLOT_WIDTH / 20;
			// Factor based on arbitrary peak to prevent bands from going off the chart. 
			// If you find a better way, submit in comments on my blog (annafilina.com)
			var FACTOR:uint = 2000 / PLOT_HEIGHT;
			var g:Graphics = this.graphics;
			g.clear();
			
			g.lineStyle(0, 0x6600CC);
			g.beginFill(0x6600CC);
			g.moveTo(0, PLOT_HEIGHT);
			
			var n:Number = 0;
			for (var i:int = 0; i < CHANNEL_LENGTH; ++i) {
				n = (bytes.readFloat() * PLOT_HEIGHT);
				g.drawRect(i * BAND_WIDTH, PLOT_HEIGHT - n / FACTOR, BAND_WIDTH - 2, n / FACTOR);
			}
			
			g.endFill();
		}
	}
}